@@ -5,10 +5,11 @@ from tftk.image.dataset import ImageDatasetUtil
 from tftk import ResumeExecutor
 from tftk import IS_ON_COLABOLATORY_WITH_GOOGLE_DRIVE, Colaboratory
 from PIL import Image
+from tftk.image.dataset import ImageUtil
 
 import matplotlib.pyplot as plt
 import numpy as np
-
+import cv2
 
 class ImageTrain():
     """画像を使用した学習を提供するユーティリティです。
@@ -68,33 +69,29 @@ class ImageTrain():
         exe = ResumeExecutor.get_instance()
 
         if IS_ON_COLABOLATORY_WITH_GOOGLE_DRIVE():
-            print("colab training with google drive")
             Colaboratory.copy_resume_data_from_google_drive()
         else:
             print("google drive is not found.")
 
         if exe.is_resumable_training()==True:
-            print("This is resume training!!")
+            # print("This is resume training!!")
             exe.resume_model(model)
             resume_val = exe.resume_values()
             initial_epoch, _, _,_  = resume_val
             initial_epoch = initial_epoch + 1
-            print("resume epoch", initial_epoch, "max_epoch", max_epoch)
+            print("resuming epoch", initial_epoch, "max_epoch", max_epoch)
         else:
             if exe.is_train_ended()==True:
                 print("Training is completed.")
                 exit()
             else:
-                print("Not resume training")
+                # print("Not resume training")
+                pass
+
 
-        """
-        optimizer='rmsprop', loss=None, metrics=None, loss_weights=None,
-        sample_weight_mode=None, weighted_metrics=None, target_tensors=None,
-        distribute=None, **kwargs
-        """
         steps_per_epoch = train_size//batch_size
         validation_steps = validation_size//batch_size
-        model.fit(
+        history = model.fit(
             train_data,
             callbacks=callbacks,
             validation_data=validation_data,
@@ -103,8 +100,9 @@ class ImageTrain():
             epochs=max_epoch, initial_epoch=initial_epoch)
 
 
-
-
+        tf.keras.backend.clear_session()
+        del optimizer,callbacks,model,train_data,validation_data
+        return history
 
 
     @classmethod
@@ -177,18 +175,24 @@ class ImageTrain():
 
         steps_per_epoch = train_size//batch_size
         validation_steps = validation_size//batch_size
-        model.fit(
+        history = model.fit(
             train_data,
             callbacks=callbacks,
             validation_data=validation_data,
             steps_per_epoch=steps_per_epoch,
             validation_steps=validation_steps,
             epochs=max_epoch, initial_epoch=initial_epoch)
+        tf.keras.backend.clear_session()
+        del optimizer,callbacks,model,train_data,validation_data
+        return history
 
-
+    
 
     @classmethod
     def show_autoencoder_results(cls, model:tf.keras.Model, dataset:tf.data.Dataset , size:int):
+        """AutoEncoderで出力した値を表示
+
+        """
      
         i = 0
         for data in dataset:
@@ -198,26 +202,93 @@ class ImageTrain():
             h = shape[0]
             w = shape[1]
             c = shape[2]
-            orig_im = Image.fromarray(img.astype(np.uint8))
+            img = img.astype(np.uint8)
+
+            if c == 1:
+                img = np.reshape(img,[h,w])
+                orig_im = Image.fromarray(img, 'L')
+            else:
+                orig_im = Image.fromarray(img)
+
             orig_im = orig_im.resize((256,256))
-            img = np.reshape(img,[1,h,w,c])
-            x = img / 255.0
+            x = np.reshape(img,[1,h,w,c])
+            x = x / 255.0
             y = model.predict(x)
             y = y * 255
             y = y.astype(np.uint8)
-            y = np.reshape(img,[h,w,c])
-            y_im = Image.fromarray(y)
+
+            if c == 1:
+                y = np.reshape(y,[h,w])
+                y_im = Image.fromarray(y, 'L')
+            else:
+                y = np.reshape(y,[h,w,c])
+                y_im = Image.fromarray(y)
+
             y_im = y_im.resize((256,256))
 
-            concat = Image.new('RGB',(512,256))
+            if c == 1:
+                concat = Image.new('L', (512,256))
+            else:
+                concat = Image.new('RGB',(512,256))
             concat.paste(orig_im,(0,0))
             concat.paste(y_im,(256,0))
-            plt.imshow(np.array(concat))
+            concat.show()
+
+            if c != 1:
+                diff = img - y
+                diff = np.abs(diff)
+                heatmap = np.mean(diff, axis=-1)
+                heatmap = np.maximum(heatmap,0)
+                heatmap /= np.max(heatmap)
+                heatmap = np.uint8(255*heatmap)
+                heatmap = cv2.applyColorMap(heatmap, cv2.COLORMAP_JET)
+
+                heatmap = ImageUtil.cv2pil(heatmap)
+                heatmap.show()
+
             i += 1
             if i == size:
                 return
 
 
+    @classmethod
+    def calucurate_reconstruction_error(cls,model:tf.keras.Model,  dataset:tf.data.Dataset, size:int):
+        dataset = dataset.batch(1)
+        i = 0
+        for data in dataset:
+            x = data["image"]
+            x = x.numpy()
+            m = x.shape[1]
+            n = x.shape[2]
+            mn = m * n
+            i = i +1
+            y = model.predict(x)
+            loss = np.mean(tf.keras.losses.MSE(x,y), axis=(1,2)) / mn
+            print(loss)
+            if i > size:
+                break
+
+    
+    @classmethod
+    def train_image_gan(cls, generator:tf.keras.Model,  discriminator:tf.keras.Model, dataset:tf.data.Dataset, gan_input_shape=(32,), **kwargs):
+
+        # discriminator コンパイル
+        discriminator_optimizer = tf.keras.optimizers.RMSprop(lr=0.008,clipvalue=1.0,decay=1e-8)
+        discriminator.compile(optimizer=discriminator_optimizer,loss='binary_crossentropy')
+
+        # 識別器はいったん学習させない
+        discriminator.trainable = False
+
+        # 
+        gan_input = tf.keras.Input(shape=gan_input_shape)
+        gan_output = discriminator(generator(gan_input))
+
+        gan = tf.keras.Model(gan_input,gan_output)
+        gan_optimizer = tf.keras.optimizers.RMSprop(lr=0.0004,clipvalue=1.0,decay=1e-8)
+        gan.compile()
+        
+
+
 """
 fit(
     x=None, y=None, batch_size=None, epochs=1, verbose=1, callbacks=None,
